IF "%CONDA_BUILD%" == "" (
    call D:\bld\nbgrader_1615406840332\work\build_env_setup.bat
)
REM ===== end generated header =====
D:\bld\nbgrader_1615406840332\_h_env\python.exe -m pip install . --no-deps -vv